@@ -7,15 +7,28 @@ import os
 import os.path
 import sys
 from hashlib import md5
+from typing import (
+    Callable,
+    Dict,
+    Hashable,
+    Iterable,
+    Optional,
+    TypeVar,
+    Union,
+    cast,
+)
 
 from filelock import FileLock
+import requests
 
 LOG = logging.getLogger(__name__)
 
 _DID_LOG_UNABLE_TO_CACHE = False
 
+T = TypeVar("T")  # pylint: disable=invalid-name
 
-def get_pkg_unique_identifier():
+
+def get_pkg_unique_identifier() -> str:
     """
     Generate an identifier unique to the python version, tldextract version, and python instance
 
@@ -31,20 +44,22 @@ def get_pkg_unique_identifier():
     tldextract_version = "tldextract-" + version
     python_env_name = os.path.basename(sys.prefix)
     # just to handle the edge case of two identically named python environments
-    python_binary_path_short_hash = hashlib.md5(sys.prefix.encode("utf-8")).hexdigest()[:6]
+    python_binary_path_short_hash = hashlib.md5(sys.prefix.encode("utf-8")).hexdigest()[
+        :6
+    ]
     python_version = ".".join([str(v) for v in sys.version_info[:-1]])
     identifier_parts = [
         python_version,
         python_env_name,
         python_binary_path_short_hash,
-        tldextract_version
+        tldextract_version,
     ]
     pkg_identifier = "__".join(identifier_parts)
 
     return pkg_identifier
 
 
-def get_cache_dir():
+def get_cache_dir() -> str:
     """
     Get a cache dir that we have permission to write to
 
@@ -62,7 +77,9 @@ def get_cache_dir():
             xdg_cache_home = os.path.join(user_home, ".cache")
 
     if xdg_cache_home is not None:
-        return os.path.join(xdg_cache_home, "python-tldextract", get_pkg_unique_identifier())
+        return os.path.join(
+            xdg_cache_home, "python-tldextract", get_pkg_unique_identifier()
+        )
 
     # fallback to trying to use package directory itself
     return os.path.join(os.path.dirname(__file__), ".suffix_cache/")
@@ -71,7 +88,7 @@ def get_cache_dir():
 class DiskCache:
     """Disk _cache that only works for jsonable values"""
 
-    def __init__(self, cache_dir, lock_timeout=20):
+    def __init__(self, cache_dir: Optional[str], lock_timeout: int = 20):
         self.enabled = bool(cache_dir)
         self.cache_dir = os.path.expanduser(str(cache_dir) or "")
         self.lock_timeout = lock_timeout
@@ -79,7 +96,7 @@ class DiskCache:
         # combined with a call to `.clear()` wont wipe someones hard drive
         self.file_ext = ".tldextract.json"
 
-    def get(self, namespace, key):
+    def get(self, namespace: str, key: Union[str, Dict[str, Hashable]]) -> object:
         """Retrieve a value from the disk cache"""
         if not self.enabled:
             raise KeyError("Cache is disabled")
@@ -88,34 +105,35 @@ class DiskCache:
         if not os.path.isfile(cache_filepath):
             raise KeyError("namespace: " + namespace + " key: " + repr(key))
         try:
+            # pylint: disable-next=unspecified-encoding
             with open(cache_filepath) as cache_file:
                 return json.load(cache_file)
         except (OSError, ValueError) as exc:
             LOG.error("error reading TLD cache file %s: %s", cache_filepath, exc)
-            raise KeyError(  # pylint: disable=raise-missing-from
-                "namespace: " + namespace + " key: " + repr(key)
-            )
+            raise KeyError("namespace: " + namespace + " key: " + repr(key)) from None
 
-    def set(self, namespace, key, value):
+    def set(
+        self, namespace: str, key: Union[str, Dict[str, Hashable]], value: object
+    ) -> None:
         """Set a value in the disk cache"""
         if not self.enabled:
-            return False
+            return
+
         cache_filepath = self._key_to_cachefile_path(namespace, key)
 
         try:
             _make_dir(cache_filepath)
+            # pylint: disable-next=unspecified-encoding
             with open(cache_filepath, "w") as cache_file:
                 json.dump(value, cache_file)
         except OSError as ioe:
             global _DID_LOG_UNABLE_TO_CACHE  # pylint: disable=global-statement
             if not _DID_LOG_UNABLE_TO_CACHE:
                 LOG.warning(
-                    (
-                        "unable to cache %s.%s in %s. This could refresh the "
-                        "Public Suffix List over HTTP every app startup. "
-                        "Construct your `TLDExtract` with a writable `cache_dir` or "
-                        "set `cache_dir=False` to silence this warning. %s"
-                    ),
+                    "unable to cache %s.%s in %s. This could refresh the "
+                    "Public Suffix List over HTTP every app startup. "
+                    "Construct your `TLDExtract` with a writable `cache_dir` or "
+                    "set `cache_dir=None` to silence this warning. %s",
                     namespace,
                     key,
                     cache_filepath,
@@ -123,9 +141,7 @@ class DiskCache:
                 )
                 _DID_LOG_UNABLE_TO_CACHE = True
 
-        return None
-
-    def clear(self):
+    def clear(self) -> None:
         """Clear the disk cache"""
         for root, _, files in os.walk(self.cache_dir):
             for filename in files:
@@ -142,7 +158,9 @@ class DiskCache:
                         if exc.errno != errno.ENOENT:
                             raise
 
-    def _key_to_cachefile_path(self, namespace, key):
+    def _key_to_cachefile_path(
+        self, namespace: str, key: Union[str, Dict[str, Hashable]]
+    ) -> str:
         namespace_path = os.path.join(self.cache_dir, namespace)
         hashed_key = _make_cache_key(key)
 
@@ -150,7 +168,13 @@ class DiskCache:
 
         return cache_path
 
-    def run_and_cache(self, func, namespace, kwargs, hashed_argnames):
+    def run_and_cache(
+        self,
+        func: Callable[..., T],
+        namespace: str,
+        kwargs: Dict[str, Hashable],
+        hashed_argnames: Iterable[str],
+    ) -> T:
         """Get a url but cache the response"""
         if not self.enabled:
             return func(**kwargs)
@@ -164,12 +188,10 @@ class DiskCache:
             global _DID_LOG_UNABLE_TO_CACHE  # pylint: disable=global-statement
             if not _DID_LOG_UNABLE_TO_CACHE:
                 LOG.warning(
-                    (
-                        "unable to cache %s.%s in %s. This could refresh the "
-                        "Public Suffix List over HTTP every app startup. "
-                        "Construct your `TLDExtract` with a writable `cache_dir` or "
-                        "set `cache_dir=False` to silence this warning. %s"
-                    ),
+                    "unable to cache %s.%s in %s. This could refresh the "
+                    "Public Suffix List over HTTP every app startup. "
+                    "Construct your `TLDExtract` with a writable `cache_dir` or "
+                    "set `cache_dir=None` to silence this warning. %s",
                     namespace,
                     key_args,
                     cache_filepath,
@@ -179,16 +201,20 @@ class DiskCache:
 
             return func(**kwargs)
 
+        # Disable lint of 3rd party (see also https://github.com/tox-dev/py-filelock/issues/102)
+        # pylint: disable-next=abstract-class-instantiated
         with FileLock(lock_path, timeout=self.lock_timeout):
             try:
-                result = self.get(namespace=namespace, key=key_args)
+                result = cast(T, self.get(namespace=namespace, key=key_args))
             except KeyError:
                 result = func(**kwargs)
-                self.set(namespace="urls", key=key_args, value=result)
+                self.set(namespace=namespace, key=key_args, value=result)
 
             return result
 
-    def cached_fetch_url(self, session, url, timeout):
+    def cached_fetch_url(
+        self, session: requests.Session, url: str, timeout: Union[float, int, None]
+    ) -> str:
         """Get a url but cache the response"""
         return self.run_and_cache(
             func=_fetch_url,
@@ -198,8 +224,7 @@ class DiskCache:
         )
 
 
-def _fetch_url(session, url, timeout):
-
+def _fetch_url(session: requests.Session, url: str, timeout: Optional[int]) -> str:
     response = session.get(url, timeout=timeout)
     response.raise_for_status()
     text = response.text
@@ -210,16 +235,12 @@ def _fetch_url(session, url, timeout):
     return text
 
 
-def _make_cache_key(inputs):
+def _make_cache_key(inputs: Union[str, Dict[str, Hashable]]) -> str:
     key = repr(inputs)
-    try:
-        key = md5(key).hexdigest()
-    except TypeError:
-        key = md5(key.encode("utf8")).hexdigest()
-    return key
+    return md5(key.encode("utf8")).hexdigest()
 
 
-def _make_dir(filename):
+def _make_dir(filename: str) -> None:
     """Make a directory if it doesn't already exist"""
     if not os.path.exists(os.path.dirname(filename)):
         try:
